@@ -20,6 +20,7 @@
 #include <QDebug>
 #include <QVariant>
 #include <QImage>
+#include <QQmlEngine>
 
 #include "akvideopacket.h"
 #include "akpacket.h"
@@ -212,6 +213,8 @@ class AkVideoPacketPrivate
         static AkVideoPacket nv21_to_rgb24(const AkVideoPacket *src, int align);
         static AkVideoPacket rgbap_to_rgb24(const AkVideoPacket *src, int align);
         static AkVideoPacket _0bgr_to_rgb24(const AkVideoPacket *src, int align);
+        static AkVideoPacket bgr0_to_rgb24(const AkVideoPacket *src, int align);
+        static AkVideoPacket bgr0_to_0rgb(const AkVideoPacket *src, int align);
 };
 
 using VideoConvertFuncs = QVector<VideoConvert>;
@@ -255,6 +258,8 @@ VideoConvertFuncs initVideoConvertFuncs()
         {AkVideoCaps::Format_nv21   , AkVideoCaps::Format_rgb24   , AkVideoPacketPrivate::nv21_to_rgb24    },
         {AkVideoCaps::Format_rgbap  , AkVideoCaps::Format_rgb24   , AkVideoPacketPrivate::rgbap_to_rgb24   },
         {AkVideoCaps::Format_0bgr   , AkVideoCaps::Format_rgb24   , AkVideoPacketPrivate::_0bgr_to_rgb24   },
+        {AkVideoCaps::Format_bgr0   , AkVideoCaps::Format_rgb24   , AkVideoPacketPrivate::bgr0_to_rgb24    },
+        {AkVideoCaps::Format_bgr0   , AkVideoCaps::Format_0rgb    , AkVideoPacketPrivate::bgr0_to_0rgb     },
     };
 
     return convert;
@@ -645,6 +650,19 @@ void AkVideoPacket::resetIndex()
     this->setIndex(-1);
 }
 
+void AkVideoPacket::registerTypes()
+{
+    qRegisterMetaType<AkVideoPacket>("AkVideoPacket");
+    qmlRegisterSingletonType<AkVideoPacket>("Ak", 1, 0, "AkVideoPacket",
+                                            [] (QQmlEngine *qmlEngine,
+                                                QJSEngine *jsEngine) -> QObject * {
+        Q_UNUSED(qmlEngine)
+        Q_UNUSED(jsEngine)
+
+        return new AkVideoPacket();
+    });
+}
+
 QDebug operator <<(QDebug debug, const AkVideoPacket &packet)
 {
     debug.nospace() << "AkVideoPacket("
@@ -1638,9 +1656,58 @@ AkVideoPacket AkVideoPacketPrivate::_0bgr_to_rgb24(const AkVideoPacket *src,
         auto dst_line = reinterpret_cast<RGB24 *>(dst.line(0, y));
 
         for (int x = 0; x < width; x++) {
-            dst_line[x].r = src_line[x].x * src_line[x].r / 255;
-            dst_line[x].g = src_line[x].x * src_line[x].g / 255;
-            dst_line[x].b = src_line[x].x * src_line[x].b / 255;
+            dst_line[x].r = src_line[x].r;
+            dst_line[x].g = src_line[x].g;
+            dst_line[x].b = src_line[x].b;
+        }
+    }
+
+    return dst;
+}
+
+AkVideoPacket AkVideoPacketPrivate::bgr0_to_rgb24(const AkVideoPacket *src, int align)
+{
+    auto caps = src->caps();
+    caps.setFormat(AkVideoCaps::Format_rgb24);
+    caps.setAlign(align);
+    AkVideoPacket dst(caps);
+    dst.copyMetadata(*src);
+    auto width = src->caps().width();
+    auto height = src->caps().height();
+
+    for (int y = 0; y < height; y++) {
+        auto src_line = reinterpret_cast<const XRGB *>(src->constLine(0, y));
+        auto dst_line = reinterpret_cast<RGB24 *>(dst.line(0, y));
+
+        for (int x = 0; x < width; x++) {
+            dst_line[x].r = src_line[x].r;
+            dst_line[x].g = src_line[x].g;
+            dst_line[x].b = src_line[x].b;
+        }
+    }
+
+    return dst;
+}
+
+AkVideoPacket AkVideoPacketPrivate::bgr0_to_0rgb(const AkVideoPacket *src, int align)
+{
+    auto caps = src->caps();
+    caps.setFormat(AkVideoCaps::Format_0rgb);
+    caps.setAlign(align);
+    AkVideoPacket dst(caps);
+    dst.copyMetadata(*src);
+    auto width = src->caps().width();
+    auto height = src->caps().height();
+
+    for (int y = 0; y < height; y++) {
+        auto src_line = reinterpret_cast<const XRGB *>(src->constLine(0, y));
+        auto dst_line = reinterpret_cast<RGBX *>(dst.line(0, y));
+
+        for (int x = 0; x < width; x++) {
+            dst_line[x].x = src_line[x].x;
+            dst_line[x].r = src_line[x].r;
+            dst_line[x].g = src_line[x].g;
+            dst_line[x].b = src_line[x].b;
         }
     }
 
