@@ -22,17 +22,15 @@
 
 QObject *DelayGrab::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new DelayGrabElement();
-
-    return nullptr;
+    return new DelayGrabElement();
 }
 
 QStringList DelayGrab::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_delaygrab.cpp"
