@@ -22,17 +22,15 @@
 
 QObject *Matrix::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new MatrixElement();
-
-    return nullptr;
+    return new MatrixElement();
 }
 
 QStringList Matrix::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_matrix.cpp"
