@@ -28,6 +28,11 @@
 #include <akfrac.h>
 #include <akcaps.h>
 
+extern "C"
+{
+    #include <libavcodec/avcodec.h>
+}
+
 #include "abstractstream.h"
 #include "clock.h"
 
@@ -53,7 +58,7 @@ class AbstractStreamPrivate
         AkFrac m_timeBase;
         AVStream *m_stream {nullptr};
         AVCodecContext *m_codecContext {nullptr};
-        AVCodec *m_codec {nullptr};
+        const AVCodec *m_codec {nullptr};
         AVDictionary *m_codecOptions {nullptr};
         QThreadPool m_threadPool;
         QMutex m_packetMutex;
@@ -64,64 +69,60 @@ class AbstractStreamPrivate
         QQueue<PacketPtr> m_packets;
         QQueue<FramePtr> m_frames;
         QQueue<SubtitlePtr> m_subtitles;
-        qint64 m_packetQueueSize {-1};
+        qint64 m_packetQueueSize {0};
         Clock *m_globalClock {nullptr};
         QFuture<void> m_packetLoopResult;
         QFuture<void> m_dataLoopResult;
         qint64 m_id {-1};
         uint m_index {0};
         AVMediaType m_mediaType {AVMEDIA_TYPE_UNKNOWN};
+        AkElement::ElementState m_state {AkElement::ElementStateNull};
+        bool m_sync {true};
         bool m_runPacketLoop {false};
-        bool m_runDataLoop {false};
+        bool m_run {false};
+        bool m_paused {false};
 
         explicit AbstractStreamPrivate(AbstractStream *self);
         void packetLoop();
+        void readPacket();
         void dataLoop();
+        void readData();
         static void deletePacket(AVPacket *packet);
         static void deleteFrame(AVFrame *frame);
         static void deleteSubtitle(AVSubtitle *subtitle);
 };
 
 AbstractStream::AbstractStream(const AVFormatContext *formatContext,
-                               uint index, qint64 id, Clock *globalClock,
+                               uint index,
+                               qint64 id,
+                               Clock *globalClock,
+                               bool sync,
                                bool noModify,
                                QObject *parent): QObject(parent)
 {
     this->d = new AbstractStreamPrivate(this);
-    this->m_paused = false;
     this->m_isValid = false;
     this->m_clockDiff = 0;
     this->m_maxData = 0;
     this->d->m_index = index;
     this->d->m_id = id;
+    this->d->m_sync = sync;
 
     this->d->m_stream = (formatContext && index < formatContext->nb_streams)?
                          formatContext->streams[index]: nullptr;
 
     this->d->m_mediaType = this->d->m_stream?
-#ifdef HAVE_CODECPAR
-                            this->d->m_stream->codecpar->codec_type:
-#else
-                            this->d->m_stream->codec->codec_type:
-#endif
-                            AVMEDIA_TYPE_UNKNOWN;
+                               this->d->m_stream->codecpar->codec_type:
+                               AVMEDIA_TYPE_UNKNOWN;
 
     this->d->m_codecContext = nullptr;
 
     if (this->d->m_stream) {
         this->d->m_codecContext = avcodec_alloc_context3(nullptr);
 
-#ifdef HAVE_CODECPAR
         if (avcodec_parameters_to_context(this->d->m_codecContext,
                                           this->d->m_stream->codecpar) < 0)
             avcodec_free_context(&this->d->m_codecContext);
-#else
-        if (avcodec_copy_context(this->d->m_codecContext,
-                                 this->d->m_stream->codec) < 0) {
-            avcodec_close(this->d->m_codecContext);
-            av_free(this->d->m_codecContext);
-        }
-#endif
     }
 
     this->d->m_codec =
@@ -130,7 +131,6 @@ AbstractStream::AbstractStream(const AVFormatContext *formatContext,
                 nullptr;
 
     this->d->m_codecOptions = nullptr;
-    this->d->m_packetQueueSize = 0;
     this->d->m_globalClock = globalClock;
 
     if (!this->d->m_codec)
@@ -165,17 +165,14 @@ AbstractStream::AbstractStream(const AVFormatContext *formatContext,
 
 AbstractStream::~AbstractStream()
 {
+    this->setState(AkElement::ElementStateNull);
+
     if (this->d->m_codecContext)
         avcodec_free_context(&this->d->m_codecContext);
 
     delete this->d;
 }
 
-bool AbstractStream::paused() const
-{
-    return this->m_paused;
-}
-
 bool AbstractStream::isValid() const
 {
     return this->m_isValid;
@@ -211,7 +208,7 @@ AVCodecContext *AbstractStream::codecContext() const
     return this->d->m_codecContext;
 }
 
-AVCodec *AbstractStream::codec() const
+const AVCodec *AbstractStream::codec() const
 {
     return this->d->m_codec;
 }
@@ -223,7 +220,32 @@ AVDictionary *AbstractStream::codecOptions() const
 
 AkCaps AbstractStream::caps() const
 {
-    return AkCaps();
+    return {};
+}
+
+bool AbstractStream::sync() const
+{
+    return this->d->m_sync;
+}
+
+qint64 AbstractStream::queueSize() const
+{
+    return this->d->m_packetQueueSize;
+}
+
+Clock *AbstractStream::globalClock()
+{
+    return this->d->m_globalClock;
+}
+
+qreal AbstractStream::clockDiff() const
+{
+    return this->m_clockDiff;
+}
+
+qreal &AbstractStream::clockDiff()
+{
+    return this->m_clockDiff;
 }
 
 void AbstractStream::packetEnqueue(AVPacket *packet)
@@ -237,8 +259,9 @@ void AbstractStream::packetEnqueue(AVPacket *packet)
         this->d->m_packets.enqueue(PacketPtr(packet,
                                              AbstractStreamPrivate::deletePacket));
         this->d->m_packetQueueSize += packet->size;
-    } else
+    } else {
         this->d->m_packets.enqueue(PacketPtr());
+    }
 
     this->d->m_packetQueueNotEmpty.wakeAll();
     this->d->m_packetMutex.unlock();
@@ -278,38 +301,24 @@ void AbstractStream::subtitleEnqueue(AVSubtitle *subtitle)
     this->d->m_dataMutex.unlock();
 }
 
-qint64 AbstractStream::queueSize()
+bool AbstractStream::decodeData()
 {
-    return this->d->m_packetQueueSize;
-}
-
-Clock *AbstractStream::globalClock()
-{
-    return this->d->m_globalClock;
-}
-
-qreal AbstractStream::clockDiff() const
-{
-    return this->m_clockDiff;
-}
-
-qreal &AbstractStream::clockDiff()
-{
-    return this->m_clockDiff;
+    return false;
 }
 
 AVMediaType AbstractStream::type(const AVFormatContext *formatContext,
                                  uint index)
 {
     return index < formatContext->nb_streams?
-#ifdef HAVE_CODECPAR
                 formatContext->streams[index]->codecpar->codec_type:
-#else
-                formatContext->streams[index]->codec->codec_type:
-#endif
                 AVMEDIA_TYPE_UNKNOWN;
 }
 
+AkElement::ElementState AbstractStream::state() const
+{
+    return this->d->m_state;
+}
+
 void AbstractStream::processPacket(AVPacket *packet)
 {
     Q_UNUSED(packet)
@@ -325,6 +334,133 @@ void AbstractStream::processData(AVSubtitle *subtitle)
     Q_UNUSED(subtitle)
 }
 
+void AbstractStream::flush()
+{
+    this->d->m_dataMutex.lock();
+    this->d->m_packets.clear();
+    this->d->m_dataMutex.unlock();
+
+    this->d->m_dataMutex.lock();
+    this->d->m_frames.clear();
+    this->d->m_subtitles.clear();
+    this->d->m_dataMutex.unlock();
+}
+
+bool AbstractStream::setState(AkElement::ElementState state)
+{
+    switch (this->d->m_state) {
+    case AkElement::ElementStateNull: {
+        if (state == AkElement::ElementStatePaused
+            || state == AkElement::ElementStatePlaying) {
+            if (!this->d->m_codecContext || !this->d->m_codec)
+                return false;
+
+            if (avcodec_open2(this->d->m_codecContext,
+                              this->d->m_codec,
+                              &this->d->m_codecOptions) < 0)
+                return false;
+
+            this->m_clockDiff = 0.0;
+            this->d->m_run = true;
+            this->d->m_runPacketLoop = true;
+            this->d->m_paused = state == AkElement::ElementStatePaused;
+            this->d->m_dataLoopResult =
+                    QtConcurrent::run(&this->d->m_threadPool,
+                                      this->d,
+                                      &AbstractStreamPrivate::dataLoop);
+            this->d->m_packetLoopResult =
+                    QtConcurrent::run(&this->d->m_threadPool,
+                                      this->d,
+                                      &AbstractStreamPrivate::packetLoop);
+            this->d->m_state = state;
+            emit this->stateChanged(state);
+
+            return true;
+        }
+
+        break;
+    }
+    case AkElement::ElementStatePaused: {
+        switch (state) {
+        case AkElement::ElementStateNull: {
+            this->d->m_runPacketLoop = false;
+            waitLoop(this->d->m_packetLoopResult);
+
+            this->d->m_run = false;
+            waitLoop(this->d->m_dataLoopResult);
+
+            if (this->d->m_codecOptions)
+                av_dict_free(&this->d->m_codecOptions);
+
+            if (this->d->m_codecContext) {
+                avcodec_close(this->d->m_codecContext);
+                this->d->m_codecContext = nullptr;
+            }
+
+            this->d->m_packets.clear();
+            this->d->m_frames.clear();
+            this->d->m_subtitles.clear();
+            this->d->m_state = state;
+            emit this->stateChanged(state);
+
+            return true;
+        }
+        case AkElement::ElementStatePlaying: {
+            this->d->m_paused = false;
+            this->d->m_state = state;
+            emit this->stateChanged(state);
+
+            return true;
+        }
+        default:
+            break;
+        }
+
+        break;
+    }
+    case AkElement::ElementStatePlaying: {
+        switch (state) {
+        case AkElement::ElementStateNull: {
+            this->d->m_runPacketLoop = false;
+            waitLoop(this->d->m_packetLoopResult);
+
+            this->d->m_run = false;
+            waitLoop(this->d->m_dataLoopResult);
+
+            if (this->d->m_codecOptions)
+                av_dict_free(&this->d->m_codecOptions);
+
+            if (this->d->m_codecContext) {
+                avcodec_close(this->d->m_codecContext);
+                this->d->m_codecContext = nullptr;
+            }
+
+            this->d->m_packets.clear();
+            this->d->m_frames.clear();
+            this->d->m_subtitles.clear();
+            this->d->m_state = state;
+            emit this->stateChanged(state);
+
+            return true;
+        }
+        case AkElement::ElementStatePaused: {
+            this->d->m_paused = true;
+            this->d->m_state = state;
+            emit this->stateChanged(state);
+
+            return true;
+        }
+        default:
+            break;
+        }
+
+        break;
+    }
+    }
+
+    return false;
+}
+
 AbstractStreamPrivate::AbstractStreamPrivate(AbstractStream *self):
     self(self)
 {
@@ -333,100 +469,124 @@ AbstractStreamPrivate::AbstractStreamPrivate(AbstractStream *self):
 void AbstractStreamPrivate::packetLoop()
 {
     while (this->m_runPacketLoop) {
-        this->m_packetMutex.lock();
-        bool gotPacket = true;
+        if (this->m_paused) {
+            QThread::msleep(500);
 
-        if (this->m_packets.isEmpty())
-            gotPacket = this->m_packetQueueNotEmpty.wait(&this->m_packetMutex,
-                                                         THREAD_WAIT_LIMIT);
-
-        PacketPtr packet;
-
-        if (gotPacket) {
-            packet = this->m_packets.dequeue();
-
-            if (packet)
-                this->m_packetQueueSize -= packet->size;
+            continue;
         }
 
-        this->m_packetMutex.unlock();
-
-        if (gotPacket) {
-            self->processPacket(packet.data());
-            emit self->notify();
-        }
-
-        if (!packet)
-            this->m_runPacketLoop = false;
+        this->readPacket();
     }
 }
 
+void AbstractStreamPrivate::readPacket()
+{
+    this->m_packetMutex.lock();
+    bool gotPacket = true;
+
+    if (this->m_packets.isEmpty())
+        gotPacket = this->m_packetQueueNotEmpty.wait(&this->m_packetMutex,
+                                                     THREAD_WAIT_LIMIT);
+
+    PacketPtr packet;
+
+    if (gotPacket && !this->m_packets.isEmpty()) {
+        packet = this->m_packets.dequeue();
+
+        if (packet)
+            this->m_packetQueueSize -= packet->size;
+    }
+
+    this->m_packetMutex.unlock();
+
+    if (gotPacket) {
+        self->processPacket(packet.data());
+        emit self->notify();
+    }
+
+    self->decodeData();
+
+    if (!packet)
+        this->m_runPacketLoop = false;
+}
+
 void AbstractStreamPrivate::dataLoop()
+{
+    while (this->m_run) {
+        if (this->m_paused) {
+            QThread::msleep(500);
+
+            continue;
+        }
+
+        this->readData();
+    }
+}
+
+void AbstractStreamPrivate::readData()
 {
     switch (self->mediaType()) {
     case AVMEDIA_TYPE_VIDEO:
-    case AVMEDIA_TYPE_AUDIO:
-        while (this->m_runDataLoop) {
-            this->m_dataMutex.lock();
-            bool gotFrame = true;
+    case AVMEDIA_TYPE_AUDIO: {
+        this->m_dataMutex.lock();
+        bool gotFrame = true;
 
-            if (this->m_frames.isEmpty())
-                gotFrame = this->m_dataQueueNotEmpty.wait(&this->m_dataMutex,
-                                                          THREAD_WAIT_LIMIT);
+        if (this->m_frames.isEmpty())
+            gotFrame = this->m_dataQueueNotEmpty.wait(&this->m_dataMutex,
+                                                      THREAD_WAIT_LIMIT);
 
-            FramePtr frame;
+        FramePtr frame;
 
-            if (gotFrame) {
-                frame = this->m_frames.dequeue();
+        if (gotFrame) {
+            frame = this->m_frames.dequeue();
 
-                if (this->m_frames.size() < self->m_maxData)
-                    this->m_dataQueueNotFull.wakeAll();
-            }
+            if (this->m_frames.size() < self->m_maxData)
+                this->m_dataQueueNotFull.wakeAll();
+        }
 
-            this->m_dataMutex.unlock();
+        this->m_dataMutex.unlock();
 
-            if (gotFrame) {
-                if (frame)
-                    self->processData(frame.data());
-                else {
-                    emit self->eof();
-                    this->m_runDataLoop = false;
-                }
+        if (gotFrame) {
+            if (frame) {
+                self->processData(frame.data());
+            } else {
+                emit self->eof();
+                this->m_run = false;
             }
         }
 
         break;
-    case AVMEDIA_TYPE_SUBTITLE:
-        while (this->m_runDataLoop) {
-            this->m_dataMutex.lock();
-            bool gotSubtitle = true;
+    }
+    case AVMEDIA_TYPE_SUBTITLE: {
+        this->m_dataMutex.lock();
+        bool gotSubtitle = true;
 
-            if (this->m_subtitles.isEmpty())
-                gotSubtitle = this->m_dataQueueNotEmpty.wait(&this->m_dataMutex,
-                                                             THREAD_WAIT_LIMIT);
+        if (this->m_subtitles.isEmpty())
+            gotSubtitle = this->m_dataQueueNotEmpty.wait(&this->m_dataMutex,
+                                                         THREAD_WAIT_LIMIT);
 
-            SubtitlePtr subtitle;
+        SubtitlePtr subtitle;
 
-            if (gotSubtitle) {
-                subtitle = this->m_subtitles.dequeue();
+        if (gotSubtitle) {
+            subtitle = this->m_subtitles.dequeue();
 
-                if (this->m_subtitles.size() < self->m_maxData)
-                    this->m_dataQueueNotFull.wakeAll();
-            }
+            if (this->m_subtitles.size() < self->m_maxData)
+                this->m_dataQueueNotFull.wakeAll();
+        }
 
-            this->m_dataMutex.unlock();
+        this->m_dataMutex.unlock();
 
-            if (gotSubtitle) {
-                if (subtitle)
-                    self->processData(subtitle.data());
-                else {
-                    emit self->eof();
-                    this->m_runDataLoop = false;
-                }
+        if (gotSubtitle) {
+            if (subtitle)
+                self->processData(subtitle.data());
+            else {
+                emit self->eof();
+                this->m_run = false;
             }
         }
 
         break;
+    }
     default:
         break;
     }
@@ -434,12 +594,18 @@ void AbstractStreamPrivate::dataLoop()
 
 void AbstractStreamPrivate::deletePacket(AVPacket *packet)
 {
+    if (!packet)
+        return;
+
     av_packet_unref(packet);
-    delete packet;
+    av_packet_free(&packet);
 }
 
 void AbstractStreamPrivate::deleteFrame(AVFrame *frame)
 {
+    if (!frame)
+        return;
+
     av_freep(&frame->data[0]);
     frame->data[0] = nullptr;
     av_frame_unref(frame);
@@ -448,79 +614,11 @@ void AbstractStreamPrivate::deleteFrame(AVFrame *frame)
 
 void AbstractStreamPrivate::deleteSubtitle(AVSubtitle *subtitle)
 {
+    if (!subtitle)
+        return;
+
     avsubtitle_free(subtitle);
     delete subtitle;
 }
 
-void AbstractStream::setPaused(bool paused)
-{
-    if (this->m_paused == paused)
-        return;
-
-    this->d->m_runDataLoop = !paused;
-
-    if (paused)
-        this->d->m_dataLoopResult.waitForFinished();
-    else
-        this->d->m_dataLoopResult =
-            QtConcurrent::run(&this->d->m_threadPool,
-                              this->d,
-                              &AbstractStreamPrivate::dataLoop);
-
-    this->m_paused = paused;
-    emit this->pausedChanged(paused);
-}
-
-void AbstractStream::resetPaused()
-{
-    this->setPaused(false);
-}
-
-bool AbstractStream::init()
-{
-    if (!this->d->m_codecContext
-        || !this->d->m_codec)
-        return false;
-
-    if (avcodec_open2(this->d->m_codecContext,
-                      this->d->m_codec,
-                      &this->d->m_codecOptions) < 0)
-        return false;
-
-    this->m_clockDiff = 0;
-    this->d->m_runPacketLoop = true;
-    this->d->m_runDataLoop = true;
-    this->d->m_packetLoopResult =
-            QtConcurrent::run(&this->d->m_threadPool,
-                              this->d,
-                              &AbstractStreamPrivate::packetLoop);
-    this->d->m_dataLoopResult =
-            QtConcurrent::run(&this->d->m_threadPool,
-                              this->d,
-                              &AbstractStreamPrivate::dataLoop);
-
-    return true;
-}
-
-void AbstractStream::uninit()
-{
-    this->d->m_runPacketLoop = false;
-    waitLoop(this->d->m_packetLoopResult);
-
-    this->d->m_runDataLoop = false;
-    waitLoop(this->d->m_dataLoopResult);
-
-    if (this->d->m_codecOptions)
-        av_dict_free(&this->d->m_codecOptions);
-
-    if (this->d->m_codecContext) {
-        avcodec_close(this->d->m_codecContext);
-        this->d->m_codecContext = nullptr;
-    }
-
-    this->d->m_packets.clear();
-    this->d->m_frames.clear();
-    this->d->m_subtitles.clear();
-}
-
 #include "moc_abstractstream.cpp"
