@@ -20,7 +20,7 @@
 #ifndef ABSTRACTSTREAM_H
 #define ABSTRACTSTREAM_H
 
-#include <QObject>
+#include <akelement.h>
 
 extern "C"
 {
@@ -40,21 +40,17 @@ class Clock;
 class AbstractStream: public QObject
 {
     Q_OBJECT
-    Q_PROPERTY(bool paused
-               READ paused
-               WRITE setPaused
-               RESET resetPaused
-               NOTIFY pausedChanged)
 
     public:
         AbstractStream(const AVFormatContext *formatContext=nullptr,
-                       uint index=0, qint64 id=-1,
+                       uint index=0,
+                       qint64 id=-1,
                        Clock *globalClock=nullptr,
+                       bool sync=true,
                        bool noModify=false,
                        QObject *parent=nullptr);
         virtual ~AbstractStream();
 
-        Q_INVOKABLE bool paused() const;
         Q_INVOKABLE bool isValid() const;
         Q_INVOKABLE uint index() const;
         Q_INVOKABLE qint64 id() const;
@@ -62,22 +58,23 @@ class AbstractStream: public QObject
         Q_INVOKABLE AVMediaType mediaType() const;
         Q_INVOKABLE AVStream *stream() const;
         Q_INVOKABLE AVCodecContext *codecContext() const;
-        Q_INVOKABLE AVCodec *codec() const;
+        Q_INVOKABLE const AVCodec *codec() const;
         Q_INVOKABLE AVDictionary *codecOptions() const;
         Q_INVOKABLE virtual AkCaps caps() const;
-        Q_INVOKABLE void packetEnqueue(AVPacket *packet);
-        Q_INVOKABLE void dataEnqueue(AVFrame *frame);
-        Q_INVOKABLE void subtitleEnqueue(AVSubtitle *subtitle);
-        Q_INVOKABLE qint64 queueSize();
+        Q_INVOKABLE bool sync() const;
+        Q_INVOKABLE qint64 queueSize() const;
         Q_INVOKABLE Clock *globalClock();
         Q_INVOKABLE qreal clockDiff() const;
         Q_INVOKABLE qreal &clockDiff();
-
-        static AVMediaType type(const AVFormatContext *formatContext,
-                                uint index);
+        Q_INVOKABLE void packetEnqueue(AVPacket *packet);
+        Q_INVOKABLE void dataEnqueue(AVFrame *frame);
+        Q_INVOKABLE void subtitleEnqueue(AVSubtitle *subtitle);
+        Q_INVOKABLE virtual bool decodeData();
+        Q_INVOKABLE static AVMediaType type(const AVFormatContext *formatContext,
+                                            uint index);
+        Q_INVOKABLE AkElement::ElementState state() const;
 
     protected:
-        bool m_paused;
         bool m_isValid;
         qreal m_clockDiff;
         int m_maxData;
@@ -90,17 +87,14 @@ class AbstractStream: public QObject
         AbstractStreamPrivate *d;
 
     signals:
-        void pausedChanged(bool paused);
+        void stateChanged(AkElement::ElementState state);
         void oStream(const AkPacket &packet);
         void notify();
-        void frameSent();
         void eof();
 
     public slots:
-        void setPaused(bool paused);
-        void resetPaused();
-        virtual bool init();
-        virtual void uninit();
+        void flush();
+        bool setState(AkElement::ElementState state);
 
         friend class AbstractStreamPrivate;
 };
