@@ -22,17 +22,15 @@
 
 QObject *Multiplex::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new MultiplexElement();
-
-    return nullptr;
+    return new MultiplexElement();
 }
 
 QStringList Multiplex::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_multiplex.cpp"
