@@ -22,17 +22,15 @@
 
 QObject *PrimariesColors::create(const QString &key, const QString &specification)
 {
+    Q_UNUSED(key)
     Q_UNUSED(specification)
 
-    if (key == AK_PLUGIN_TYPE_ELEMENT)
-        return new PrimariesColorsElement();
-
-    return nullptr;
+    return new PrimariesColorsElement();
 }
 
 QStringList PrimariesColors::keys() const
 {
-    return QStringList();
+    return {};
 }
 
 #include "moc_primariescolors.cpp"
